<?php

declare(strict_types=1);

namespace Drupal\presigned_url\Drush\Commands;

use Drupal\presigned_url\PresignedUrlService;
use Drush\Attributes as CLI;
use Drush\Commands\DrushCommands;
use Consolidation\OutputFormatters\StructuredData\RowsOfFields;
use Drush\Commands\AutowireTrait;

/**
 * Drush commands for presigned URLs.
 */
class PresignedUrlCommands extends DrushCommands {

  use AutowireTrait;

  /**
   * Constructs a new PresignedUrlCommands object.
   */
  public function __construct(
    protected readonly PresignedUrlService $presignedUrlService,
  ) {}

  /**
   * Creates a presigned URL for a private file.
   */
  #[CLI\Command(name: 'presigned-url:sign', aliases: ['pus'])]
  #[CLI\Argument(name: 'uri', description: 'The URI of the private file (e.g., private://example.pdf)')]
  #[CLI\Option(
    name: 'expires',
    description: 'Expiration time in seconds (default: 900)',
    suggestedValues: [300, 900, 1800, 3600],
  )]
  #[CLI\Usage(name: 'drush presigned-url:sign private://example.pdf', description: 'Create a presigned URL with default expiration')]
  #[CLI\Usage(name: 'drush pus private://example.pdf --expires=3600', description: 'Create a presigned URL that expires in 1 hour')]
  #[CLI\ValidateFileExists(argName: 'uri')]
  public function sign(string $uri, int $expires = 900): RowsOfFields {
    $url = $this->presignedUrlService->createPresignedUrl($uri, $expires);

    if ($url === NULL) {
      throw new \InvalidArgumentException('Could not generate presigned URL. Ensure the file exists and is in the private file system.');
    }

    $rows = [
      [
        'uri' => $uri,
        'expires' => $expires,
        'expires_at' => date('Y-m-d H:i:s', time() + $expires),
        'url' => $url,
      ],
    ];

    return new RowsOfFields($rows);
  }

}
