<?php

declare(strict_types=1);

namespace Drupal\presigned_url\Plugin\rest\resource;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\presigned_url\PresignedUrlService;
use Drupal\rest\Plugin\ResourceBase;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\file\FileRepositoryInterface;
use Drupal\rest\Attribute\RestResource;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Wunderwerk\HttpApiUtils\HttpApiValidationTrait;
use Wunderwerk\JsonApiError\JsonApiErrorResponse;

/**
 * Provides a resource for creating presigned URLs.
 */
#[RestResource(
  id: 'presigned_url',
  label: new TranslatableMarkup('Presigned URL'),
  uri_paths: [
    'create' => '/api/v1/presigned-url',
  ],
)]
class PresignedUrlResource extends ResourceBase {

  use HttpApiValidationTrait;

  /**
   * The schema for the request body.
   */
  protected array $schema = [
    'type' => 'object',
    'properties' => [
      'uri' => [
        'type' => 'string',
      ],
      'expires' => [
        'type' => 'integer',
        'minimum' => 1,
        'maximum' => 24 * 60 * 60,
      ],
    ],
    'required' => ['uri'],
  ];

  /**
   * Constructs a PresignedUrlResource object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param array $serializer_formats
   *   The available serialization formats.
   * @param \Psr\Log\LoggerInterface $logger
   *   A logger instance.
   * @param \Drupal\Core\Session\AccountProxyInterface $currentUser
   *   The current user.
   * @param \Drupal\presigned_url\PresignedUrlService $presignedUrlService
   *   The presigned URL service.
   * @param \Drupal\file\FileRepositoryInterface $fileRepository
   *   The file repository service.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    array $serializer_formats,
    LoggerInterface $logger,
    protected AccountProxyInterface $currentUser,
    protected PresignedUrlService $presignedUrlService,
    protected FileRepositoryInterface $fileRepository,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->getParameter('serializer.formats'),
      $container->get('logger.factory')->get('rest'),
      $container->get('current_user'),
      $container->get(PresignedUrlService::class),
      $container->get(FileRepositoryInterface::class),
    );
  }

  /**
   * Responds to POST requests.
   */
  public function post(Request $request): JsonResponse {
    $jsonBody = $request->getContent();
    $data = Json::decode($jsonBody);

    $result = $this->validateArray($data, $this->schema);
    if (!$result->isValid()) {
      return $result->getResponse();
    }

    // File does not exist.
    if (!file_exists($data['uri'])) {
      return JsonApiErrorResponse::fromError(
        status: 404,
        code: 'file_not_found',
        title: 'File not found',
      );
    }

    // File does not have a database record.
    $file = $this->fileRepository->loadByUri($data['uri']);
    if (!$file) {
      return JsonApiErrorResponse::fromError(
        status: 404,
        code: 'file_not_found',
        title: 'File not found',
      );
    }

    // User does not have access to the file.
    if (!$file->access('download', $this->currentUser)) {
      return JsonApiErrorResponse::fromError(
        status: 403,
        code: 'access_denied',
        title: 'Access denied',
        detail: 'Presigned URL can only be created for files that the current user has access to.',
      );
    }

    try {
      $url = $this->presignedUrlService->createPresignedUrl($data['uri'], $data['expires'] ?? PresignedUrlService::DEFAULT_EXPIRATION);

      return new JsonResponse(['url' => $url]);
    }
    catch (\Exception $e) {
      return JsonApiErrorResponse::fromError(
        status: 500,
        code: 'internal_server_error',
        title: 'Internal server error',
        detail: $e->getMessage(),
      );
    }
  }

}
