<?php

declare(strict_types=1);

namespace Drupal\presigned_url\Signature;

use Drupal\Core\Site\Settings;

/**
 * Generates a signature for a presigned URL.
 */
class PresignedUrlSignatureGenerator {

  public function __construct(
    protected readonly Settings $settings,
  ) {}

  /**
   * Generates a signature for the presigned URL.
   */
  public function generateSignature(string $url): string {
    $privateKey = $this->settings->getHashSalt();

    // Parse the URL into components.
    $urlParts = parse_url($url);

    // Normalize path - keep encoded characters as-is.
    $path = $urlParts['path'] ?? '';
    $path = $this->normalizePath($path);
    $urlParts['path'] = $path;

    // Parse and sort query parameters if they exist.
    if (isset($urlParts['query'])) {
      parse_str($urlParts['query'], $queryParams);

      // Remove signature parameter if it exists.
      unset($queryParams[PresignedUrlSignatureInterface::QUERY_PARAM_SIGNATURE]);

      // Sort query parameters recursively to handle array parameters.
      $this->recursiveKsort($queryParams);

      // Rebuild query string, preserving array parameter format.
      $urlParts['query'] = http_build_query($queryParams, '', '&', PHP_QUERY_RFC3986);
    }

    // Create signature string starting with the path.
    $signatureString = $urlParts['path'];

    // Add query parameters if they exist.
    if (isset($urlParts['query'])) {
      $signatureString .= '?' . $urlParts['query'];
    }

    // Add fragment if it exists.
    if (isset($urlParts['fragment'])) {
      $signatureString .= '#' . $urlParts['fragment'];
    }

    return hash_hmac('sha256', $signatureString, $privateKey);
  }

  /**
   * Normalizes a URL path.
   */
  protected function normalizePath(string $path): string {
    // Remove consecutive slashes.
    $path = preg_replace('#/+#', '/', $path);
    // Remove /./ sequences.
    $path = preg_replace('#/\./#', '/', $path);
    // Remove trailing slash except for root.
    $path = rtrim($path, '/');
    if (empty($path)) {
      $path = '/';
    }
    return $path;
  }

  /**
   * Recursively sorts an array by key.
   */
  protected function recursiveKsort(array &$array): void {
    ksort($array);
    foreach ($array as &$value) {
      if (is_array($value)) {
        $this->recursiveKsort($value);
      }
    }
  }

}
