<?php

declare(strict_types=1);

namespace Drupal\Tests\presigned_url\Kernel;

use Drupal\Core\File\FileExists;
use Drupal\Core\Site\Settings;
use Drupal\KernelTests\Core\File\FileTestBase;
use Drupal\presigned_url\Exception\PresignedUrlException;
use Drupal\presigned_url\PresignedUrlService;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\HttpKernelInterface;

/**
 * Tests the presigned URL file download functionality.
 *
 * @group presigned_url
 * @coversDefaultClass \Drupal\presigned_url\Hook\FileHooks
 */
class PresignedUrlFileDownloadTest extends FileTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'presigned_url',
    'system',
    'file',
    'user',
    'field',
    'node',
  ];

  /**
   * The presigned URL service.
   */
  protected PresignedUrlService $presignedUrlService;

  /**
   * The HTTP kernel.
   */
  protected HttpKernelInterface $httpKernel;

  /**
   * The test file URI.
   */
  protected string $privateFileUri = 'private://test.txt';

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create a new settings object with a known hash salt
    // for predictable testing.
    new Settings([
      'hash_salt' => 'test-hash-salt',
      'file_private_path' => $this->siteDirectory . '/private',
    ]);

    $this->installConfig(['system']);
    $this->installSchema('file', ['file_usage']);
    $this->installEntitySchema('file');
    $this->installEntitySchema('node');
    $this->installEntitySchema('user');

    // Create the Article content type with a file field.
    $node_type = \Drupal::entityTypeManager()->getStorage('node_type')
      ->create([
        'type' => 'article',
        'name' => 'Article',
      ]);
    $node_type->save();

    // Create a file field.
    $field_storage = \Drupal::entityTypeManager()->getStorage('field_storage_config')
      ->create([
        'field_name' => 'field_file',
        'entity_type' => 'node',
        'type' => 'file',
        'settings' => [
          'uri_scheme' => 'private',
        ],
      ]);
    $field_storage->save();

    $field_config = \Drupal::entityTypeManager()->getStorage('field_config')
      ->create([
        'field_storage' => $field_storage,
        'bundle' => 'article',
        'label' => 'File',
        'settings' => [
          'file_directory' => '',
          'file_extensions' => 'txt',
        ],
      ]);
    $field_config->save();

    // Create a file entity for the test file.
    /** @var \Drupal\file\FileRepositoryInterface $file_repository */
    $file_repository = $this->container->get('file.repository');
    $file_entity = $file_repository->writeData(
      'Test file content',
      $this->privateFileUri,
      FileExists::Replace,
    );
    $file_entity->setPermanent();
    $file_entity->save();

    // Create a node that references the file.
    $node = \Drupal::entityTypeManager()->getStorage('node')
      ->create([
        'type' => 'article',
        'title' => 'Test Article',
        'field_file' => [
          'target_id' => $file_entity->id(),
          'display' => 1,
        ],
      ]);
    $node->save();

    $this->presignedUrlService = $this->container->get(PresignedUrlService::class);
    $this->httpKernel = $this->container->get('http_kernel');
  }

  /**
   * Tests downloading a file with a valid presigned URL.
   *
   * @covers ::handlePresignedFileDownload
   */
  public function testFileDownloadWithValidPresignedUrl(): void {
    // Create a presigned URL.
    $url = $this->presignedUrlService->createPresignedUrl($this->privateFileUri);
    $this->assertNotNull($url, 'Presigned URL should be created');

    // Create and handle the request.
    $request = Request::create($url);
    $response = $this->httpKernel->handle($request);

    // Assert successful download.
    $this->assertEquals(200, $response->getStatusCode());
  }

  /**
   * Tests downloading a file with an expired presigned URL.
   *
   * @covers ::handlePresignedFileDownload
   */
  public function testFileDownloadWithExpiredPresignedUrl(): void {
    // Create a presigned URL that expires immediately.
    $url = $this->presignedUrlService->createPresignedUrl($this->privateFileUri, 0);
    $this->assertNotNull($url, 'Presigned URL should be created');

    // Sleep for 1 second to ensure expiration.
    sleep(1);

    // Create and handle the request.
    $request = Request::create($url);
    $response = $this->httpKernel->handle($request);

    // Assert access denied.
    $this->assertEquals(403, $response->getStatusCode());
  }

  /**
   * Tests downloading a file with a tampered presigned URL.
   *
   * @covers ::handlePresignedFileDownload
   */
  public function testFileDownloadWithTamperedPresignedUrl(): void {
    // Create a presigned URL.
    $url = $this->presignedUrlService->createPresignedUrl($this->privateFileUri);
    $this->assertNotNull($url, 'Presigned URL should be created');

    // Tamper with the URL by changing the signature.
    $tamperedUrl = str_replace(
      'signature=',
      'signature=tampered',
      $url
    );

    // Create and handle the request.
    $request = Request::create($tamperedUrl);
    $response = $this->httpKernel->handle($request);

    // Assert access denied.
    $this->assertEquals(403, $response->getStatusCode());
  }

  /**
   * Tests downloading a non-private file.
   *
   * @covers ::handlePresignedFileDownload
   */
  public function testFileDownloadWithNonPrivateFile(): void {
    // Create a public file.
    $public_directory = $this->siteDirectory . '/files';
    $test_file = $public_directory . '/public-test.txt';
    file_put_contents($test_file, 'Public test file content');
    $public_uri = 'public://public-test.txt';

    // Try to create a presigned URL (should return null)
    $this->expectException(PresignedUrlException::class);
    $this->expectExceptionMessage('The file is not in the private file system.');
    $this->presignedUrlService->createPresignedUrl($public_uri);
  }

  /**
   * Tests downloading a non-existent file.
   *
   * @covers ::handlePresignedFileDownload
   */
  public function testFileDownloadWithNonExistentFile(): void {
    // Try to create a presigned URL for non-existent file.
    $this->expectException(PresignedUrlException::class);
    $this->expectExceptionMessage('The file does not exist.');
    $this->presignedUrlService->createPresignedUrl('private://non-existent.txt');

    // Try to access the non-existent file.
    $request = Request::create('/system/files/non-existent.txt');
    $response = $this->httpKernel->handle($request);

    // Assert file not found.
    $this->assertEquals(404, $response->getStatusCode());
  }

  /**
   * Tests downloading a file with missing URL parameters.
   *
   * @covers ::handlePresignedFileDownload
   */
  public function testFileDownloadWithMissingParameters(): void {
    // Create requests with missing parameters.
    $requests = [
    // Missing signature.
      Request::create('/system/files/test.txt?algorithm=HMAC-SHA256&expires=900&date=20240101T000000Z'),
    // Missing algorithm.
      Request::create('/system/files/test.txt?signature=abc&expires=900&date=20240101T000000Z'),
    // Missing expires.
      Request::create('/system/files/test.txt?algorithm=HMAC-SHA256&signature=abc&date=20240101T000000Z'),
    // Missing date.
      Request::create('/system/files/test.txt?algorithm=HMAC-SHA256&signature=abc&expires=900'),
    ];

    foreach ($requests as $request) {
      $response = $this->httpKernel->handle($request);
      $this->assertEquals(403, $response->getStatusCode(), 'Access should be denied for requests with missing parameters');
    }
  }

}
