<?php

declare(strict_types=1);

namespace Drupal\Tests\presigned_url\Kernel;

use Drupal\Core\File\FileExists;
use Drupal\Core\Site\Settings;
use Drupal\KernelTests\Core\File\FileTestBase;
use Drupal\presigned_url\PresignedUrlService;
use Drupal\rest\Entity\RestResourceConfig;
use Drupal\Tests\EntityTrait;
use Drupal\Tests\user\Traits\UserCreationTrait;
use Drupal\user\UserInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Tests the presigned URL REST resource.
 *
 * @group presigned_url
 * @coversDefaultClass \Drupal\presigned_url\Plugin\rest\resource\PresignedUrlResource
 */
class PresignedUrlResourceTest extends FileTestBase {

  use EntityTrait;
  use UserCreationTrait {
    checkPermissions as drupalCheckPermissions;
    createAdminRole as drupalCreateAdminRole;
    createRole as drupalCreateRole;
    createUser as drupalCreateUser;
    grantPermissions as drupalGrantPermissions;
    setCurrentUser as drupalSetCurrentUser;
    setUpCurrentUser as drupalSetUpCurrentUser;
  }

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'presigned_url',
    'rest',
    'serialization',
    'system',
    'file',
    'user',
    'field',
    'node',
  ];

  /**
   * The presigned URL service.
   */
  protected PresignedUrlService $presignedUrlService;

  /**
   * The test file URI.
   */
  protected string $privateFileUri = 'private://test.txt';

  /**
   * The REST resource path.
   */
  protected string $resourcePath = '/api/v1/presigned-url';

  /**
   * The user.
   */
  protected UserInterface $user;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create a new settings object with a known hash salt
    // for predictable testing.
    new Settings([
      'hash_salt' => 'test-hash-salt',
      'file_private_path' => $this->siteDirectory . '/private',
    ]);

    $this->installConfig(['system', 'rest']);
    $this->installSchema('file', ['file_usage']);
    $this->installEntitySchema('file');
    $this->installEntitySchema('node');
    $this->installEntitySchema('user');

    // Create the Article content type with a file field.
    $node_type = \Drupal::entityTypeManager()->getStorage('node_type')
      ->create([
        'type' => 'article',
        'name' => 'Article',
      ]);
    $node_type->save();

    // Create a file field.
    $field_storage = \Drupal::entityTypeManager()->getStorage('field_storage_config')
      ->create([
        'field_name' => 'field_file',
        'entity_type' => 'node',
        'type' => 'file',
        'settings' => [
          'uri_scheme' => 'private',
        ],
      ]);
    $field_storage->save();

    $field_config = \Drupal::entityTypeManager()->getStorage('field_config')
      ->create([
        'field_storage' => $field_storage,
        'bundle' => 'article',
        'label' => 'File',
        'settings' => [
          'file_directory' => '',
          'file_extensions' => 'txt',
        ],
      ]);
    $field_config->save();

    // Create a file entity for the test file.
    /** @var \Drupal\file\FileRepositoryInterface $file_repository */
    $file_repository = $this->container->get('file.repository');
    $file_entity = $file_repository->writeData(
      'Test file content',
      $this->privateFileUri,
      FileExists::Replace,
    );
    $file_entity->setPermanent();
    $file_entity->save();

    // Create a node that references the file.
    $node = \Drupal::entityTypeManager()->getStorage('node')
      ->create([
        'type' => 'article',
        'title' => 'Test Article',
        'field_file' => [
          'target_id' => $file_entity->id(),
          'display' => 1,
        ],
      ]);
    $node->save();

    $this->presignedUrlService = $this->container->get(PresignedUrlService::class);

    RestResourceConfig::create([
      'id' => 'presigned_url',
      'plugin_id' => 'presigned_url',
      'granularity' => RestResourceConfig::RESOURCE_GRANULARITY,
      'configuration' => [
        'methods' => ['POST'],
        'formats' => ['json'],
        'authentication' => ['cookie'],
      ],
    ])->save();

    $this->user = $this->drupalCreateUser([
      'restful post presigned_url',
    ]);
    $this->drupalSetCurrentUser($this->user);
  }

  /**
   * Tests creating a presigned URL with invalid schema.
   */
  public function testInvalidSchema(): void {
    // Test missing required URI.
    $request = Request::create(
      $this->resourcePath,
      'POST',
      [],
      [],
      [],
      ['CONTENT_TYPE' => 'application/json'],
      json_encode(['expires' => 3600])
    );
    $response = $this->container->get('http_kernel')->handle($request);
    $this->assertEquals(422, $response->getStatusCode());
    $data = json_decode($response->getContent(), TRUE);
    $this->assertEquals('validation_error', $data['errors'][0]['code']);

    // Test invalid expires value.
    $request = Request::create(
      $this->resourcePath,
      'POST',
      [],
      [],
      [],
      ['CONTENT_TYPE' => 'application/json'],
      json_encode(['uri' => $this->privateFileUri, 'expires' => -1])
    );
    $response = $this->container->get('http_kernel')->handle($request);
    $this->assertEquals(422, $response->getStatusCode());
    $data = json_decode($response->getContent(), TRUE);
    $this->assertEquals('validation_error', $data['errors'][0]['code']);
  }

  /**
   * Tests creating a presigned URL for a non-existent file.
   */
  public function testFileNotExisting(): void {
    $request = Request::create(
      $this->resourcePath,
      'POST',
      [],
      [],
      [],
      ['CONTENT_TYPE' => 'application/json'],
      json_encode(['uri' => 'private://non-existent.txt'])
    );
    $response = $this->container->get('http_kernel')->handle($request);
    $this->assertEquals(404, $response->getStatusCode());
    $data = json_decode($response->getContent(), TRUE);
    $this->assertEquals('file_not_found', $data['errors'][0]['code']);
  }

  /**
   * Tests creating a presigned URL for a file without database record.
   */
  public function testFileNotInDatabase(): void {
    // Create a file on the filesystem but not in the database.
    $file_path = $this->siteDirectory . '/private/test-no-db.txt';
    file_put_contents($file_path, 'Test content');

    $request = Request::create(
      $this->resourcePath,
      'POST',
      [],
      [],
      [],
      ['CONTENT_TYPE' => 'application/json'],
      json_encode(['uri' => 'private://test-no-db.txt'])
    );
    $response = $this->container->get('http_kernel')->handle($request);
    $this->assertEquals(404, $response->getStatusCode());
    $data = json_decode($response->getContent(), TRUE);
    $this->assertEquals('file_not_found', $data['errors'][0]['code']);
  }

  /**
   * Tests creating a presigned URL successfully.
   */
  public function testSuccessfulPresignedUrl(): void {
    $request = Request::create(
      $this->resourcePath,
      'POST',
      [],
      [],
      [],
      ['CONTENT_TYPE' => 'application/json'],
      json_encode(['uri' => $this->privateFileUri, 'expires' => 3600])
    );
    $response = $this->container->get('http_kernel')->handle($request);
    $this->assertEquals(200, $response->getStatusCode());
    $data = json_decode($response->getContent(), TRUE);
    $this->assertArrayHasKey('url', $data);
    $this->assertNotEmpty($data['url']);
  }

}
