<?php

declare(strict_types=1);

namespace Drupal\Tests\presigned_url\Kernel;

use Drupal\Core\Site\Settings;
use Drupal\KernelTests\Core\File\FileTestBase;
use Drupal\presigned_url\Exception\PresignedUrlException;
use Drupal\presigned_url\PresignedUrlService;
use Drupal\presigned_url\Signature\PresignedUrlSignatureInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Tests the presigned URL service.
 *
 * @group presigned_url
 * @coversDefaultClass \Drupal\presigned_url\PresignedUrlService
 */
class PresignedUrlServiceTest extends FileTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'presigned_url',
    'system',
    'file',
    'user',
  ];

  /**
   * The presigned URL service.
   */
  protected PresignedUrlService $presignedUrlService;

  /**
   * The test file URI.
   */
  protected string $privateFileUri;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create a new settings object with a known hash salt
    // for predictable testing.
    new Settings([
      'hash_salt' => 'test-hash-salt',
      'file_private_path' => $this->siteDirectory . '/private',
    ]);

    $this->installConfig(['system']);
    $this->installSchema('file', ['file_usage']);
    $this->installEntitySchema('file');
    $this->installEntitySchema('user');

    // Create private files directory.
    $private_directory = $this->siteDirectory . '/private';
    $test_file = $private_directory . '/test.txt';
    file_put_contents($test_file, 'Test file content');
    $this->privateFileUri = 'private://test.txt';

    $this->presignedUrlService = $this->container->get(PresignedUrlService::class);
  }

  /**
   * Tests creating a presigned URL for a private file.
   *
   * @covers ::createPresignedUrl
   */
  public function testCreatePresignedUrl(): void {
    $url = $this->presignedUrlService->createPresignedUrl($this->privateFileUri);

    $this->assertNotNull($url, 'Presigned URL should be created for private file');
    $this->assertStringContainsString('system/files/test.txt', $url, 'URL should contain the file path');
    $this->assertStringContainsString(PresignedUrlSignatureInterface::QUERY_PARAM_ALGORITHM . '=HMAC-SHA256', $url);
    $this->assertStringContainsString(PresignedUrlSignatureInterface::QUERY_PARAM_EXPIRES . '=900', $url);
    $this->assertStringContainsString(PresignedUrlSignatureInterface::QUERY_PARAM_DATE, $url);
    $this->assertStringContainsString(PresignedUrlSignatureInterface::QUERY_PARAM_SIGNATURE, $url);
  }

  /**
   * Tests creating a presigned URL for a non-private file.
   *
   * @covers ::createPresignedUrl
   */
  public function testCreatePresignedUrlNonPrivate(): void {
    $this->expectException(PresignedUrlException::class);
    $this->expectExceptionMessage('The file is not in the private file system.');
    $this->presignedUrlService->createPresignedUrl('public://test.txt');
  }

  /**
   * Tests creating a presigned URL for a non-existent file.
   *
   * @covers ::createPresignedUrl
   */
  public function testCreatePresignedUrlNonExistent(): void {
    $this->expectException(PresignedUrlException::class);
    $this->expectExceptionMessage('The file does not exist.');
    $this->presignedUrlService->createPresignedUrl('private://non-existent.txt');
  }

  /**
   * Tests validating a presigned URL.
   *
   * @covers ::validatePresignedUrl
   */
  public function testValidatePresignedUrl(): void {
    // Create a presigned URL.
    $url = $this->presignedUrlService->createPresignedUrl($this->privateFileUri);
    $this->assertNotNull($url, 'Presigned URL should be created');

    // Create a request from the URL.
    $request = Request::create($url);

    // Validate the URL.
    $isValid = $this->presignedUrlService->validatePresignedUrl($request);
    $this->assertTrue($isValid, 'Presigned URL should be valid');
  }

  /**
   * Tests validating an expired presigned URL.
   *
   * @covers ::validatePresignedUrl
   */
  public function testValidateExpiredPresignedUrl(): void {
    // Create a presigned URL that expires immediately.
    $url = $this->presignedUrlService->createPresignedUrl($this->privateFileUri, 0);
    $this->assertNotNull($url, 'Presigned URL should be created');

    // Sleep for 1 second to ensure expiration.
    sleep(1);

    // Create a request from the URL.
    $request = Request::create($url);

    // Validate the URL.
    $isValid = $this->presignedUrlService->validatePresignedUrl($request);
    $this->assertFalse($isValid, 'Expired presigned URL should be invalid');
  }

  /**
   * Tests validating a tampered presigned URL.
   *
   * @covers ::validatePresignedUrl
   */
  public function testValidateTamperedPresignedUrl(): void {
    // Create a presigned URL.
    $url = $this->presignedUrlService->createPresignedUrl($this->privateFileUri);
    $this->assertNotNull($url, 'Presigned URL should be created');

    // Tamper with the URL by changing the signature.
    $tamperedUrl = str_replace(
      'signature=',
      'signature=tampered',
      $url
    );

    // Create a request from the tampered URL.
    $request = Request::create($tamperedUrl);

    // Validate the URL.
    $isValid = $this->presignedUrlService->validatePresignedUrl($request);
    $this->assertFalse($isValid, 'Tampered presigned URL should be invalid');
  }

  /**
   * Tests validating a presigned URL with missing parameters.
   *
   * @covers ::validatePresignedUrl
   */
  public function testValidatePresignedUrlMissingParams(): void {
    // Create requests with missing parameters.
    $requests = [
    // Missing signature.
      Request::create('/test?algorithm=HMAC-SHA256&expires=900&date=20240101T000000Z'),
    // Missing algorithm.
      Request::create('/test?signature=abc&expires=900&date=20240101T000000Z'),
    // Missing expires.
      Request::create('/test?algorithm=HMAC-SHA256&signature=abc&date=20240101T000000Z'),
    // Missing date.
      Request::create('/test?algorithm=HMAC-SHA256&signature=abc&expires=900'),
    ];

    foreach ($requests as $request) {
      $isValid = $this->presignedUrlService->validatePresignedUrl($request);
      $this->assertFalse($isValid, 'Presigned URL with missing parameters should be invalid');
    }
  }

}
