<?php

declare(strict_types=1);

namespace Drupal\Tests\presigned_url\Kernel;

use Drupal\Core\Site\Settings;
use Drupal\KernelTests\KernelTestBase;
use Drupal\presigned_url\Signature\PresignedUrlSignatureGenerator;
use Drupal\presigned_url\Signature\PresignedUrlSignatureInterface;

/**
 * Tests the presigned URL signature generation.
 *
 * @group presigned_url
 * @coversDefaultClass \Drupal\presigned_url\Signature\PresignedUrlSignatureGenerator
 */
class PresignedUrlSignatureGeneratorTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['presigned_url'];

  /**
   * The signature generator service.
   */
  protected PresignedUrlSignatureGenerator $signatureGenerator;

  /**
   * The test hash salt.
   */
  protected string $hashSalt = 'test-hash-salt';

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create a new settings object with a known hash salt
    // for predictable testing.
    new Settings([
      'hash_salt' => $this->hashSalt,
    ]);

    $this->signatureGenerator = new PresignedUrlSignatureGenerator(\Drupal::service('settings'));
  }

  /**
   * Tests signature generation with just a path.
   *
   * @covers ::generateSignature
   */
  public function testGenerateSignatureWithPath(): void {
    $url = '/test/path';
    $signature1 = $this->signatureGenerator->generateSignature($url);
    $signature2 = $this->signatureGenerator->generateSignature($url);

    $this->assertEquals($signature1, $signature2, 'Multiple signature generations for the same path should be identical');
  }

  /**
   * Tests signature generation with query parameters.
   *
   * @covers ::generateSignature
   */
  public function testGenerateSignatureWithQueryParams(): void {
    // Test that parameter order doesn't matter.
    $url1 = '/test/path?b=2&a=1&c=3';
    $url2 = '/test/path?a=1&b=2&c=3';
    $signature1 = $this->signatureGenerator->generateSignature($url1);
    $signature2 = $this->signatureGenerator->generateSignature($url2);

    $this->assertEquals($signature1, $signature2, 'Signatures should be identical regardless of parameter order');

    // Test that signature parameter is excluded.
    $url1 = '/test/path?b=2&' . PresignedUrlSignatureInterface::QUERY_PARAM_SIGNATURE . '=abc&a=1';
    $url2 = '/test/path?a=1&b=2';
    $signature1 = $this->signatureGenerator->generateSignature($url1);
    $signature2 = $this->signatureGenerator->generateSignature($url2);

    $this->assertEquals($signature1, $signature2, 'Signature parameter should not affect the generated signature');
  }

  /**
   * Tests signature generation with URL fragments.
   *
   * @covers ::generateSignature
   */
  public function testGenerateSignatureWithFragment(): void {
    $url1 = '/test/path#section1';
    $url2 = '/test/path#section1';
    $url3 = '/test/path#different';

    $signature1 = $this->signatureGenerator->generateSignature($url1);
    $signature2 = $this->signatureGenerator->generateSignature($url2);
    $signature3 = $this->signatureGenerator->generateSignature($url3);

    $this->assertEquals($signature1, $signature2, 'Identical URLs with fragments should generate identical signatures');
    $this->assertNotEquals($signature1, $signature3, 'Different fragments should generate different signatures');
  }

  /**
   * Tests signature generation with all URL components.
   *
   * @covers ::generateSignature
   */
  public function testGenerateSignatureWithAllComponents(): void {
    // Test URLs with different parameter order but same content.
    $url1 = '/test/path?c=3&a=1&b=2#section1';
    $url2 = '/test/path?a=1&b=2&c=3#section1';
    $signature1 = $this->signatureGenerator->generateSignature($url1);
    $signature2 = $this->signatureGenerator->generateSignature($url2);

    $this->assertEquals($signature1, $signature2, 'Parameter order should not affect signatures');

    // Test with and without signature parameter.
    $url1 = '/test/path?c=3&' . PresignedUrlSignatureInterface::QUERY_PARAM_SIGNATURE . '=abc&a=1&b=2#section1';
    $url2 = '/test/path?a=1&b=2&c=3#section1';
    $signature1 = $this->signatureGenerator->generateSignature($url1);
    $signature2 = $this->signatureGenerator->generateSignature($url2);

    $this->assertEquals($signature1, $signature2, 'Signature parameter should be excluded from signature generation');

    // Test that any change affects the signature.
    // Changed c=3 to c=4.
    $url3 = '/test/path?a=1&b=2&c=4#section1';
    $signature3 = $this->signatureGenerator->generateSignature($url3);
    $this->assertNotEquals($signature1, $signature3, 'Changed parameter values should generate different signatures');
  }

  /**
   * Tests signature generation with path normalization edge cases.
   *
   * @covers ::generateSignature
   */
  public function testGenerateSignaturePathNormalization(): void {
    // Test double slashes.
    $url1 = '/test//path';
    $url2 = '/test/path';
    $signature1 = $this->signatureGenerator->generateSignature($url1);
    $signature2 = $this->signatureGenerator->generateSignature($url2);
    $this->assertEquals($signature1, $signature2, 'Double slashes should produce different signatures');

    // Test dot segments.
    $url3 = '/test/./path';
    $signature3 = $this->signatureGenerator->generateSignature($url3);
    $this->assertEquals($signature2, $signature3, 'Path with dot segments should produce different signatures');

    // Test trailing slash.
    $url4 = '/test/path/';
    $signature4 = $this->signatureGenerator->generateSignature($url4);
    $this->assertEquals($signature2, $signature4, 'Trailing slash should produce different signatures');
  }

  /**
   * Tests signature generation with empty components.
   *
   * @covers ::generateSignature
   */
  public function testGenerateSignatureEmptyComponents(): void {
    // Test empty path.
    $url1 = '';
    $signature1 = $this->signatureGenerator->generateSignature($url1);
    $this->assertNotEmpty($signature1, 'Empty path should still generate a signature');

    // Test empty query parameter.
    $url2 = '/test/path?';
    $signature2 = $this->signatureGenerator->generateSignature($url2);
    $this->assertNotEquals(
      $this->signatureGenerator->generateSignature('/test/path'),
      $signature2,
      'Empty query parameter should produce different signature'
    );

    // Test empty fragment.
    $url3 = '/test/path#';
    $signature3 = $this->signatureGenerator->generateSignature($url3);
    $this->assertNotEquals(
      $this->signatureGenerator->generateSignature('/test/path'),
      $signature3,
      'Empty fragment should produce different signature'
    );
  }

  /**
   * Tests signature generation with URL encoding variations.
   *
   * @covers ::generateSignature
   */
  public function testGenerateSignatureUrlEncoding(): void {
    // Test URL encoded slashes.
    $url1 = '/test%2Fpath';
    $url2 = '/test/path';
    $signature1 = $this->signatureGenerator->generateSignature($url1);
    $signature2 = $this->signatureGenerator->generateSignature($url2);
    $this->assertNotEquals($signature1, $signature2, 'URL encoded slashes should produce different signatures');

    // Test URL encoded spaces.
    $url3 = '/test%20path';
    $url4 = '/test path';
    $signature3 = $this->signatureGenerator->generateSignature($url3);
    $signature4 = $this->signatureGenerator->generateSignature($url4);
    $this->assertNotEquals($signature3, $signature4, 'URL encoded spaces should produce different signatures');
  }

  /**
   * Tests signature generation with complex query parameters.
   *
   * @covers ::generateSignature
   */
  public function testGenerateSignatureQueryParameters(): void {
    // Test array parameters.
    $url1 = '/test?arr[]=1&arr[]=2';
    $url2 = '/test?arr[0]=1&arr[1]=2';
    $signature1 = $this->signatureGenerator->generateSignature($url1);
    $signature2 = $this->signatureGenerator->generateSignature($url2);
    $this->assertEquals($signature1, $signature2, 'Array parameter formats should be normalized to the same signature');

    // Test array parameters.
    $url1 = '/test?arr[]=1&arr[]=2';
    $url2 = '/test?arr[1]=1&arr[0]=2';
    $signature1 = $this->signatureGenerator->generateSignature($url1);
    $signature2 = $this->signatureGenerator->generateSignature($url2);
    $this->assertNotEquals($signature1, $signature2, 'Array parameter order should affect the signature');

    // Test encoded parameter values.
    $url3 = '/test?param=hello%20world';
    $url4 = '/test?param=hello world';
    $signature3 = $this->signatureGenerator->generateSignature($url3);
    $signature4 = $this->signatureGenerator->generateSignature($url4);
    $this->assertEquals($signature3, $signature4, 'URL encoded parameter values should produce the same signature');

    // Test parameter name case sensitivity.
    $url5 = '/test?Param=value';
    $url6 = '/test?param=value';
    $signature5 = $this->signatureGenerator->generateSignature($url5);
    $signature6 = $this->signatureGenerator->generateSignature($url6);
    $this->assertNotEquals($signature5, $signature6, 'Parameter name case should produce different signatures');
  }

  /**
   * Tests signature security properties.
   *
   * @covers ::generateSignature
   */
  public function testSignatureSecurity(): void {
    $url = '/test/path?param=value';

    // Test signature length (should be 64 characters for SHA-256)
    $signature = $this->signatureGenerator->generateSignature($url);
    $this->assertEquals(64, strlen($signature), 'SHA-256 signature should be 64 characters long');

    // Test signature differs with empty components.
    $signatures = [];
    $signatures[] = $this->signatureGenerator->generateSignature('/test/path');
    $signatures[] = $this->signatureGenerator->generateSignature('/test/path?');
    $signatures[] = $this->signatureGenerator->generateSignature('/test/path#');
    $signatures[] = $this->signatureGenerator->generateSignature('/test/path?#');

    // Ensure all signatures are unique.
    $this->assertEquals(count($signatures), count(array_unique($signatures)),
      'Different URL forms should produce unique signatures');

    // Test Unicode normalization.
    // é as single character.
    $url3 = '/test/café';
    // E + combining acute accent.
    $url4 = '/test/cafe' . hex2bin('CC81');
    $signature3 = $this->signatureGenerator->generateSignature($url3);
    $signature4 = $this->signatureGenerator->generateSignature($url4);
    $this->assertNotEquals($signature3, $signature4,
      'Different Unicode normalizations should produce different signatures');
  }

}
